/**
 * @returns {Modeling}
 */
function getCurrentModeler() {
  return dmnModeler.getActiveViewer().get("modeling");
}

/**
 *
 * @param {VerificationEntry}  verificationEntry
 * @param {VerificationFix} fix
 */
function performVerificationFix(verificationEntry, fix, id, $callerButton) {
  console.log("FIX", fix);
  // highlight row
  cleanHighlightFunction();
  $('.verification-container').removeClass('highlight-soft');
  $('#ver-entry-' + id).addClass('highlight-soft');
  for (let i = 0; i < fix.actions.length; i++) {
    switch (fix.actions[i].actionType) {
      case 'SHOW':
        performVerificationFixSHOW(verificationEntry, fix.actions[i]);
        break;
      case 'UPDATE':
        alert("Action not defined: " + fix);
        $callerButton.css('display', 'none');
        break;
      case 'CREATE':
        performVerificationFixCREATE(verificationEntry, fix.actions[i]);
        $callerButton.css('display', 'none');
        break;
      case 'DELETE':
        performVerificationFixDELETE(verificationEntry, fix.actions[i]);
        $callerButton.css('display', 'none');
        break;
      default:
        alert("Action not defined: " + fix);
    }
    performHighlightFunction();
  }
}

/**
 *
 * @param {VerificationEntry} verificationEntry
 * @param {Action} fixAction
 */
function performVerificationFixSHOW(verificationEntry, fixAction) {
  switch (fixAction.actionScope) {
    case 'RULE':
      fixSHOW_RULE(verificationEntry, fixAction);
      break;
    case 'INPUT_ENTRY':
      fixSHOW_INPUT_ENTRY(verificationEntry, fixAction);
      break;
    case 'OUTPUT_ENTRY':
      fixSHOW_OUTPUT_ENTRY(verificationEntry, fixAction);
      break;
    case 'INPUT_DATA':
      fixSHOW_INPUT_DATA(verificationEntry, fixAction);
      break;
    case 'INPUT_COLUMN':
      fixSHOW_INPUT_COLUMN(verificationEntry, fixAction);
      break;
    case 'OUTPUT_COLUMN':
      fixSHOW_OUTPUT_COLUMN(verificationEntry, fixAction);
      break;
    case 'DECISION_TABLE':
      fixSHOW_DECISION_TABLE(verificationEntry, fixAction);
      break;
    case 'DECISION':
      fixSHOW_DECISION(verificationEntry, fixAction);
      break;
    default:
      alert("ACTION: " + fix.actions[i].actionType + ' -> '
          + fix.actions[i].actionScope); // TODO
  }
}

/**
 *
 * @param {VerificationEntry} verificationEntry
 * @param {Action} fixAction
 */
function fixSHOW_RULE(verificationEntry, fixAction) {
  highlightRules(verificationEntry.elements);
}

/**
 *
 * @param {VerificationEntry} verificationEntry
 * @param {Action} fixAction
 */
function fixSHOW_INPUT_ENTRY(verificationEntry, fixAction) {
  highlightDataEntries(verificationEntry.elements, 'inputEntryId');
}

/**
 *
 * @param {VerificationEntry} verificationEntry
 * @param {Action} fixAction
 */
function fixSHOW_INPUT_DATA(verificationEntry, fixAction) {
  highlightInputData(verificationEntry.elements);
}

/**
 *
 * @param {VerificationEntry} verificationEntry
 * @param {Action} fixAction
 */
function fixSHOW_OUTPUT_ENTRY(verificationEntry, fixAction) {
  highlightDataEntries(verificationEntry.elements, 'outputEntryId');
}

/**
 *
 * @param {VerificationEntry} verificationEntry
 * @param {Action} fixAction
 */
function fixSHOW_INPUT_COLUMN(verificationEntry, fixAction) {
  highlightColumns(verificationEntry.elements, 'inputId');
}

/**
 *
 * @param {VerificationEntry} verificationEntry
 * @param {Action} fixAction
 */
function fixSHOW_OUTPUT_COLUMN(verificationEntry, fixAction) {
  highlightColumns(verificationEntry.elements, 'outputId');
}

/**
 *
 * @param {VerificationEntry} verificationEntry
 * @param {Action} fixAction
 */
function fixSHOW_DECISION_TABLE(verificationEntry, fixAction) {
  highlightDecision(verificationEntry.elements);
}

/**
 *
 * @param {VerificationEntry} verificationEntry
 * @param {Action} fixAction
 */
function fixSHOW_DECISION(verificationEntry, fixAction) {
  highlightDecision(verificationEntry.elements);
}

/**
 *
 * @param {VerificationEntry} verificationEntry
 * @param {Action} fixAction
 */
function performVerificationFixCREATE(verificationEntry, fixAction) {
  switch (fixAction.actionScope) {
    case 'RULE':
      createRule(verificationEntry, fixAction);
      break;
    case 'INPUT_ENTRY':
    case 'OUTPUT_ENTRY':
    case 'INPUT_DATA':
    case 'INPUT_COLUMN':
    case 'OUTPUT_COLUMN':
    case 'DECISION_TABLE':
    case 'DECISION':
    default:
      alert("ACTION undefined: " + fix.actions[i].actionType + ' -> '
          + fix.actions[i].actionScope);
  }
}

/**
 *
 * @param {VerificationEntry} verificationEntry
 * @param {Action} fixAction
 */
function performVerificationFixDELETE(verificationEntry, fixAction) {
  switch (fixAction.actionScope) {
    case 'RULE':
      deleteRule(verificationEntry, fixAction);
      break;
    case 'INPUT_ENTRY':
    case 'OUTPUT_ENTRY':
    case 'INPUT_DATA':
    case 'INPUT_COLUMN':
    case 'OUTPUT_COLUMN':
    case 'DECISION_TABLE':
    case 'DECISION':
    default:
      alert("ACTION undefined: " + fix.actions[i].actionType + ' -> '
          + fix.actions[i].actionScope);
  }
}

/**
 *
 * @param {VerificationEntry}  verificationEntry
 * @param {Action} fixAction
 */
function createRule(verificationEntry, fixAction) {
  openViewWithId(verificationEntry.elements[0].identifier['decisionId']);
  const modeler = getCurrentModeler();
  const rule = modeler.addRow({type: "dmn:DecisionRule"});
  const {cells} = rule;
  for (const [key, value] of Object.entries(fixAction.actionValues)) {
    let cell = getCellByColId(key, cells);
    if (cell) {
      modeler.editCell(getCellByColId(key, cells), value);
    }
  }
}

/**
 *
 * @param {VerificationEntry}  verificationEntry
 * @param {Action} fixAction
 */
function deleteRule(verificationEntry, fixAction) {
  openViewWithId(verificationEntry.elements[0].identifier['decisionId']);
  const modeler = getCurrentModeler();
  for (const [key, value] of Object.entries(fixAction.actionValues)) {
    if (key === 'ruleId') {
      modeler.removeRow(modeler._sheet._elementRegistry.get(value));
    }
  }
}

// get a rule:
// dmnModeler.getActiveViewer().get('modeling')._sheet._elementRegistry.get('DecisionRule_049xiav')

function getCellByColId(colId, cells) {
  for (let i = 0; i < cells.length; i++) {
    if (cells[i].col.id === colId) {
      return cells[i];
    }
  }
  return undefined;
}
